// JavaScript Document
// author: Broes.nl
// this code may freely be used, changed and redistributed
// when you're literally copying and using the code, please refer to the author
var debug = false;
moment.locale("nl")

var GUI = function() {

    // url to brandstofprijzen
    var brandstofprijzenUrl = "http://www.athloncarlease.com/athlon-nl/self-service/brandstofprijs/";

    // default private properties
    var itemsPerPage = 3;
    
    // private methods
    function setup () { // perform setup steps
        if(!window.widget) {
            document.body.style.width = '358px';
            document.getElementById('front').style.height = (80 + itemsPerPage * 46 + 40 - 10) + "px";
            document.getElementById('back').style.height = (80 + itemsPerPage * 46 + 40 - 10) + "px";
        }
        
        // initialize globals
        var gDoneButton = new AppleGlassButton(document.getElementById("doneButton"), "Done", hidePrefs); 
        var gInfoButton = new AppleInfoButton(document.getElementById("infoButton"), document.getElementById("front"), "white", "white", showPrefs);
        
        // UI event listeners
        // button receives mouseDown
        document.getElementById('btnSearch').addEventListener('mousedown', mousedownSearch, false);
        // dropdown(s) selection has changed
        document.getElementById('afstand').addEventListener('change', afstandSelectChange, false);
        document.getElementById('brandstoftype').addEventListener('change', brandstofSelectChange, false);
        document.getElementById('chkUpdate').addEventListener('change', updatePrefChange, false);
        // form is submitted
        document.getElementById('frmBrandstof').addEventListener('submit', submitSearchForm, false);
        // dropdown(s) receives/loses focus
        document.getElementById('afstand').addEventListener('focus', function() { document.getElementById('selectAfstand').style.backgroundPosition = "center bottom"; }, false);
        document.getElementById('brandstoftype').addEventListener('focus', function() { document.getElementById('selectBrandstoftype').style.backgroundPosition = "center bottom"; }, false);
        document.getElementById('afstand').addEventListener('blur', function() { document.getElementById('selectAfstand').style.backgroundPosition = "center top"; }, false);
        document.getElementById('brandstoftype').addEventListener('blur', function() { document.getElementById('selectBrandstoftype').style.backgroundPosition = "center top"; }, false);
        // url clicks
        document.getElementById('broesurl').addEventListener('click', function() { openSite("http://www.broes.nl"); }, false);
        document.getElementById('athlonurl').addEventListener('click', function() { openSite(brandstofprijzenUrl) }, false);
        
        // initialise notification and spinner overlays
        document.getElementById('notifications').style.display = "none";
        document.getElementById('loadingOverlay').style.display = "none";
        LoadingSpinner.init("loadingSpinnerCanvas");
        BroesUpdater.init("updatepanel");
        
        // only perform when widget
        if (window.widget) {
            widget.onhide = onWidgetHide;
            widget.onshow = onWidgetShow;
        }
        
        getPreferences();
        
        // clean up
        delete setup;
    };
    
    function getPreferences () {
        if (window.widget) {
            var formPref = widget.preferenceForKey("form");
            if (formPref && formPref.length > 0) {
                var prefForm = JSON.parse(formPref);
                document.getElementById('postcode').value = prefForm.zipcode;
                document.getElementById('brandstoftype').selectedIndex = prefForm.brandstoftypeIndex;
                brandstofSelectChange();
                document.getElementById('afstand').selectedIndex = prefForm.afstandIndex;
                afstandSelectChange();
            }
            
            var listPref = widget.preferenceForKey("gasstations");
            if (listPref && listPref.length > 0) {
                var prefGSArray = JSON.parse(listPref);
                var prefGSList = GasstationList();
                for (var i = 0; i < prefGSArray.length; i++) {
                    prefGSList.addItem(Gasstation(prefGSArray[i].name, prefGSArray[i].address, prefGSArray[i].distance, prefGSArray[i].price, prefGSArray[i].date, prefGSArray[i].city, prefGSArray[i].zipcode));
                }
                updateList(prefGSList);
            }
        }
    }
    
    function onWidgetHide () {
    }
    
    function onWidgetShow () {
        BroesUpdater.isItTimeToUpdate();
    }
    
    function scrollToPage (pageNumber) {
        
        var frontResults = document.getElementById('resultContainer');
        
        // if page clicked is not the currentpage
        if (pageNumber * 338 != frontResults.scrollLeft) {
            // stop and remove existing Animator
            if (frontResults.pageScrollAnimator) {
                frontResults.pageScrollAnimator.stop();
                delete frontResults.pageScrollAnimator;
            }
            
            // animation easing function
            function easeInOut (currentValue, startValue, finishValue) {
                var returnValue = currentValue; // failsafe
                var delta = (finishValue - startValue);
                var percentage = (1 - (finishValue - currentValue)/delta);
                if (percentage < 0.5) {
                    returnValue = (startValue + Math.pow(percentage * 2, 3) * delta/2 );
                }
                else {
                    returnValue = (finishValue - Math.pow((1 - percentage) * 2, 3) * delta/2 );
                }
                return Math.ceil(returnValue);
            }
            
            // animation function to scroll
            function scrollAnimation (currentAnimation, currentValue, isStartValue, isFinishValue) {
                frontResults.scrollLeft = easeInOut(currentValue, this.from, this.to);
            }

            // loop over pagebuttons to set 'selected'
            var pageListItems = document.getElementById('paginationList').getElementsByTagName('li');
            for (var i = 0; i < pageListItems.length; i++) {
                if (i != pageNumber) { pageListItems.item(i).className = ""; }
                else { pageListItems.item(i).className = "selected"; }
            }
            
            // attach new Animator and start animation
            frontResults.pageScrollAnimator = new AppleAnimator(500, 40, frontResults.scrollLeft, pageNumber * 338, scrollAnimation); // (duration, step duration, start, finish, handler) || 50ms is 20fps
            frontResults.pageScrollAnimator.start();
            
        }
    }
    
    function submitSearchForm (e) {
        e.preventDefault();
        e.stopPropagation();
        FuelRates.getRates(brandstofprijzenUrl);
    }
    
    function mousedownSearch (e) {
        var btn = e.currentTarget;
        btn.className = 'down';

        function removeListeners() {
            btn.className = '';
            btn.removeEventListener('mouseup', mouseupSearch, false);
            btn.removeEventListener('mouseout', removeListeners, false);
        }

        function mouseupSearch (e_up) {
            removeListeners();
            FuelRates.getRates(brandstofprijzenUrl);
        }
        
        btn.addEventListener('mouseup', mouseupSearch, false);
        btn.addEventListener('mouseout', removeListeners, false);
        
        e.preventDefault();
        e.stopPropagation();
    }
    
    function afstandSelectChange (e) {
        var afstandEl = document.getElementById('afstand');
        document.getElementById('selectAfstandLabel').innerHTML = afstandEl[afstandEl.selectedIndex].text;
    }
    
    function brandstofSelectChange (e) {
        var brandstoftypeEl = document.getElementById('brandstoftype');
        document.getElementById('selectBrandstoftypeLabel').innerHTML = brandstoftypeEl[brandstoftypeEl.selectedIndex].text;
    }
    
    function updatePrefChange (e) {
        var isChecked = document.getElementById('chkUpdate').checked;
        if (window.widget) {
            widget.setPreferenceForKey(isChecked, "allowUpdateChecks");
        }
        if (isChecked) {
            BroesUpdater.isItTimeToUpdate(true);
        }
    }
    
    function showPrefs () {
        var front = document.getElementById("front");
        var back = document.getElementById("back");
        
        if (window.widget) {
            widget.prepareForTransition("ToBack");        // freezes the widget so that you can change it without the user noticing
        }
        
        front.style.display="none";        // hide the front
        back.style.display="block";        // show the back
        
        if (window.widget) {
            setTimeout(function() { widget.performTransition(); }, 0);        // and flip the widget over
        }
    };
    
    function hidePrefs () {
        var front = document.getElementById("front");
        var back = document.getElementById("back");
        
        if (window.widget) {
            widget.prepareForTransition("ToFront");        // freezes the widget and prepares it for the flip back to the front
        }
        
        back.style.display="none";            // hide the back
        front.style.display="block";        // show the front
        
        if (window.widget) {
            setTimeout(function() { widget.performTransition(); }, 0);        // and flip the widget back to the front
        }
        
    };
    
    function openSite (url) {
        var _url = url || "http://www.broes.nl";
        if (window.widget) { widget.openURL(_url); }
    }
    
    function clearResultList () {
        // remove all resultpages
        var resultScroller = document.getElementById('resultScroller');
        while (resultScroller.hasChildNodes()) {
            resultScroller.removeChild(resultScroller.lastChild);
        }
        
        // remove all pagebuttons
        var pageList = document.getElementById('paginationList');
        while (pageList.hasChildNodes()) {
            pageList.removeChild(pageList.lastChild);
        }
    }
    
    function updateList(listGasstations) {
        document.getElementById('resultContainer').scrollLeft = 0;
        
        clearResultList();
        var resultScroller = document.getElementById('resultScroller');
        var pageList = document.getElementById('paginationList');
        
        // pagina's maken per aantal itemsPerPage
        var currentPage;
        var currentResult;
        var currentGasstation;
        var numberResultPages = 0;
        for (var i = 0; i < listGasstations.getLength(); i++) {
            // create no more than 14 pages
            if (i / itemsPerPage < 14) {
                currentGasstation = listGasstations.getItem(i);
                // create new page for every itemsPerPage items
                if (i%itemsPerPage == 0) {
                    currentPage = resultScroller.appendChild(document.createElement("div"));
                    currentPage.className = "resultPage";
                    // add pagebutton for every page
                    pageList.appendChild(document.createElement("li"));
                    pageList.lastChild.addEventListener('click', function(pageNumber) { return function() { scrollToPage(pageNumber); }; } (numberResultPages), false);
                    if (numberResultPages == 0) {
                        pageList.lastChild.className = "selected";
                    }
                    numberResultPages++;
                }
                // add result data to currentPage
                currentResult = currentPage.appendChild(document.createElement("div"));
                currentResult.className = "result";
                currentResult.appendChild(document.createElement("div"));
                currentResult.lastChild.className = "logo " + currentGasstation.getLogoClass();
                currentResult.appendChild(document.createElement("p"));
                currentResult.lastChild.className = "address";
                currentResult.lastChild.innerHTML = currentGasstation.getCity() + "<br>" + currentGasstation.getAddress();
                currentResult.lastChild.addEventListener('click', function(mapsUrl) { return function() { openSite(mapsUrl); }; } (currentGasstation.getMapsUrl()), false);
                currentResult.appendChild(document.createElement("p"));
                currentResult.lastChild.className = "name";
                currentResult.lastChild.innerHTML = currentGasstation.getName();
                currentResult.appendChild(document.createElement("p"));
                currentResult.lastChild.className = "distance";
                currentResult.lastChild.innerHTML = currentGasstation.getDistance() + " <span>km</span>";
                currentResult.appendChild(document.createElement("p"));
                currentResult.lastChild.className = "price";
                currentResult.lastChild.innerHTML = "<span>€ </span>" + currentGasstation.getPrice().replace(/\./, ",");
                currentResult.appendChild(document.createElement("p"));
                currentResult.lastChild.className = "date";
                currentResult.lastChild.innerHTML = currentGasstation.getDate();                
            }
        }
        
        // set width and height according to results and settings
        resultScroller.style.width = (numberResultPages * 358) + "px";
        resultScroller.style.height = (itemsPerPage * 46) + "px";
        pageList.style.width = (numberResultPages * 20) + "px";
    }
    
    function noUpdateFound(message) {
        clearResultList();
        var notif = document.getElementById('notifications');
        notif.innerHTML = message || "Geen tankstations gevonden";
        notif.style.display = "block";
        LoadingSpinner.stop();
        document.getElementById('loadingOverlay').style.display = "none";
    }
    
    // public methods
    return {
        setup: setup,
        openSite: openSite,
        updateList: updateList,
        noUpdateFound: noUpdateFound
    };
}();

var FuelRates = function() {
    
    // private methods
    function getRates() {
        // Request naar https://www.athlon.com/api/fuelwatchapi/point?postalCode=5614 voor lat/lon
        // Request naar https://www.athlon.com/api/fuelwatchapi/search?distance=5&fuelType=4665&latitude=51.4272742&longitude=5.4929446 voor resultaten
        
        var zipCode = document.getElementById('postcode').value;
        var fuelType = document.getElementById('brandstoftype')[document.getElementById('brandstoftype').selectedIndex].value;
        var distance = document.getElementById('afstand')[document.getElementById('afstand').selectedIndex].value;
        
        // Validate user input
        if (!/^\d{4}$/.test(zipCode)) {
            GUI.noUpdateFound("Gebruik 4 cijfers voor Postcode!");
        }
        
        var searchPrefs = JSON.stringify({"zipcode": zipCode, "afstandIndex": document.getElementById('afstand').selectedIndex, "brandstoftypeIndex": document.getElementById('brandstoftype').selectedIndex});
        if (window.widget) { widget.setPreferenceForKey(searchPrefs, "form") };

        document.getElementById('notifications').style.display = "none";
        LoadingSpinner.start();
        document.getElementById('loadingOverlay').style.display = "block";
            
        getPointLatLon(zipCode, fuelType, distance);

    } // end getRates()
    
    function getPointLatLon(zipCode, fuelType, distance) {
        console.debug("getPointLatLon", zipCode, fuelType, distance);
        
        var req = new XMLHttpRequest();
            
        req.onreadystatechange = function () {
            if (this.readyState == 4) {
                if (this.status == 200 && (this.getResponseHeader("Content-Type") || "").indexOf("application/json") >= 0) {
                    getGasStations(JSON.parse(this.responseText), fuelType, distance);
                }
                else {
                    GUI.noUpdateFound("Kan geen verbinding maken");
                }
            }
        };
        
        req.open("GET", "https://www.athlon.com/api/fuelwatchapi/point?postalCode=" + zipCode);
        req.send();
    }
    
    function getGasStations(latLon, fuelType, distance) {
        console.debug("getGasStations", JSON.stringify(latLon), fuelType, distance);
        
        var req = new XMLHttpRequest();
        
        req.onreadystatechange = function () {
            if (this.readyState === 4) {
                if (this.status === 200 && (this.getResponseHeader("Content-Type") || "").indexOf("application/json") >= 0) {
                    parseFuelRates(JSON.parse(req.responseText));
                }
                else {
                    GUI.noUpdateFound("Kan geen verbinding maken");
                }
                
                LoadingSpinner.stop();
                document.getElementById('loadingOverlay').style.display = "none";
            }
        }
        
        req.open("GET", "https://www.athlon.com/api/fuelwatchapi/search?distance=" + distance + "&fuelType=" + fuelType + "&latitude=" + latLon.latitude + "&longitude=" + latLon.longitude);
        req.send();
    }
    
    function parseFuelRates(stationsResponse) {
        console.debug("parseFuelRates", JSON.stringify(stationsResponse));

        stationsResponse = stationsResponse || {};
    
        var listGasstations = GasstationList();

        listGasstations.addItems((stationsResponse.results || []).map(function (station) {
            // name, address, distance, price, date, city, zipcode
            return Gasstation(station.name, station.street + " " + station.number, station.distance, station.listFuelPrice[0].price, station.listFuelPrice[0].date, station.city, station.postalCode)
        }));
        
        listGasstations.sort("price", true);

        console.debug(JSON.stringify(listGasstations));
        // listGasstations.addItem();

        if (listGasstations.getLength() > 0) {
            if (window.widget) {
                widget.setPreferenceForKey(JSON.stringify(listGasstations), "gasstations");
            }
            GUI.updateList(listGasstations);
        }
        else { GUI.noUpdateFound(); }
    }; // end parseFuelRates()
    
    // public methods
    return {
        getRates: getRates
    }
}();

function Gasstation(name, address, distance, price, date, city, zipcode) {
    
    // processing on creation
    var logoClass = "general";
    var namePattern = /\b(?:amigo|anwb|argos|autofood|avia|bp|brand\soil|calpam|elan|esso|firezone|gulf|makro|q8|shell|tamoil|tango|texaco|tinq|total)\b/gi;
    var nameMatches = name.match(namePattern);
    if (nameMatches && nameMatches.length > 0) {
        logoClass = nameMatches[0].replace(/\s/gi,"").toLowerCase();
    }
    
    var _city = city.toLowerCase();
    var _mapsUrl = "http://maps.google.nl/maps?q=" + encodeURIComponent(address + ", " + zipcode.match(/\d{4}/i) + " " + _city);
    
    var _date = moment(date, "DD/MM/YYYY").format("D MMM YYYY");
    
    
    // public functions
    return {
        getName: function() { return name },
        getAddress: function() { return address },
        getDistance: function() { return distance },
        getPrice: function() { return price },
        getDate: function() { return _date },
        getLogoClass: function() { return logoClass },
        getCity: function() { return _city },
        getMapsUrl: function() { return _mapsUrl },
        toJSON: function(key) { return { "name": name, "address": address, "distance": distance, "price": price, "date": date, "city": city, "zipcode": zipcode }; }
    };
}

function GasstationList() {
    var listArray = [];
    var _ascending = false;
    
    // sorting functions
    function byDistance(a,b) {
        var ret = a.getDistance() - b.getDistance();
        if (_ascending) { return ret; }
        else { return -ret; }
    }
    
    function byPrice(a,b) {
        var ret = a.getPrice() - b.getPrice();
        if (_ascending) { return ret; }
        else { return -ret; }
    }

    return {
        getItem: function(index) { return listArray[index]; },
        addItem: function(gasstationItem) { listArray.push(gasstationItem); },
        addItems: function(gasstationItems) {
            listArray = listArray.concat(gasstationItems);
        },
        sort: function(sortBy, ascending) {
            _ascending = ascending || false;
            switch(sortBy) {
                case "distance":
                    listArray.sort(byDistance);
                    break;
                case "price":
                default:
                    listArray.sort(byPrice);
                    break;
            }
        },
        getLength: function() { return listArray.length; },
        toJSON: function(key) {
            var jsonArray = [];
            for (var i = 0; i < listArray.length; i++) {
                jsonArray.push(listArray[i].toJSON(null));
            }
            return jsonArray;
        }
    };
}

String.prototype.trim = function() {
    return this.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
};

String.prototype.encodeHTMLSafe = function() {
    var safeText = this.replace(/&/g,"&amp;");
    safeText = safeText.replace(/</g,"&lt;");
    return safeText;
};

String.prototype.decodeHTMLSafe = function() {
    var unSafeText = this.replace(/&lt;/g,"<");
    unSafeText = unSafeText.replace(/&gt;/g,">");
    unSafeText = unSafeText.replace(/&amp;/g,"&");
    return unSafeText;
};